% fun:             objective function to be minimized
% lb_percentage:   lower bound of optimal weights
% ub_percentage:   upper bound of optimal weights
% sum_of_weights:  sum of portfolio weights
% trials:          number of random starting points
% n:               number of assets

function [x] = moptimize(fun,lb_percentage,ub_percentage,sum_of_weights,trials,n)
try
    
    options=optimset(...
        'largescale','off',...
        'algorithm','sqp',...  % or 'active-set'
        'display','off',...
        'tolfun',10^-3,...     % Termination tolerance on the function value.
        'tolcon',10^-3,...     % The algorithm treats a point to be feasible if constraint violation is less than TolCon.
        'tolX',10^-3,...       % Termination tolerance on x
        'maxiter',300,...
        'maxfunevals',n*500);
    
    A   = [];                      % A*x<b
    b   = [];
    
    Aeq = ones(1,n);               % Aeq*x=beq
    beq = sum_of_weights;          % sum of the weights
    
    lb1 = lb_percentage*ones(n,1); % lb1<=x<=ub1
    ub1 = ub_percentage*ones(n,1); % lb1<=x<=ub1
    
    % 1/N vector of weights
    % fval:objective function value at solution (fval=fun(x))
    % ef<0:no solution
    [x,fval,ef] = fmincon(fun,repmat(sum_of_weights/n,n,1),A,b,Aeq,beq,lb1,ub1,[],options);
    if (ef<0)
       x = repmat(sum_of_weights/n,n,1);
    end
    
    % #trials of random starting points, choose the best point
    for citac = 1:trials
        x0             = rand(n,1);
        x0             = x0./sum(x0)+(sum_of_weights-1)/n;
        [x_,fval_,ef_] = fmincon(fun,x0,A,b,Aeq,beq,lb1,ub1,[],options);
        
        % try to find the global minimum
        if (fval_<fval)&&(ef_>=0) % if find better solution
            x    = x_;
            fval = fval_; 
            %ef   = ef_;
        end
    end
    
    % final precise optimization
    options=optimset(...
        'largescale','off',...
        'algorithm','sqp',...
        'display','off',...
        'tolfun',10^-8,...     % Termination tolerance on the function value.
        'tolcon',10^-8,...     % The algorithm treats a point to be feasible if constraint violation is less than TolCon.
        'tolX',10^-4,...       % Termination tolerance on x
        'maxiter',700,...
        'maxfunevals',n*1000);
    
    x = fmincon(fun,x,A,b,Aeq,beq,lb1,ub1,[],options);
    
catch ME
    errorMessage = sprintf('Error in function moptimize.\n Error Message:%s\n', ME.message);
    fprintf(1, errorMessage);
    save error;
end
end