% | Fit a t copula using method of moments
% |
% |----------------------------------------------------------------------
% |
% | [rho nu] = tcopulafit(u)
% |
% |----------------------------------------------------------------------
% |
% | INPUT:
% |
% |   u       matrix of data in range (0,1)
% |
% |----------------------------------------------------------------------
% |
% | OUTPUT:
% |
% |   rho     correlation matrix of copula
% |   nu      degrees of freedom of copula
% |
% |----------------------------------------------------------------------
% |
% | REFERENCES:
% |
% |   "Quantitative Risk Management" by McNeil, Frey and Embrechts,
% |                                 2005, Princton University Press.
% |
% |----------------------------------------------------------------------

function [rho, nu]=tcopulafit(u)
    [n,d]=size(u);

    rho=sin((0.5*pi)*corr(u,'type','Kendall'));

    R=chol(rho);

    mu=fminbnd(@negloglike,0,1);

    nu=invcdf(mu);

%---------------------------------------------------------------------
    function nll = negloglike(mu_)
        nu_=invcdf(mu_);
        t = tinv(u,nu_);
        tRinv = t / R;
        nll = - n*gammaln((nu_+d)/2) + n*d*gammaln((nu_+1)/2) - n*(d-1)*gammaln(nu_/2) ...
              + n*sum(log(abs(diag(R)))) ...
              + ((nu_+d)/2)*sum(log(1+sum(tRinv.^2, 2)./nu_)) ...
              - ((nu_+1)/2)*sum(sum(log(1+t.^2./nu_),2),1);
    end

%---------------------------------------------------------------------
    function q=invcdf(p)
        if p<=0.9
            q=(1000/9)*p;
        else
            q=100*(1-10*(p-0.9))^-5;
        end
    end

end