function [multi_ret,trun_ret1,trun_ret2] = copula_sim(X,n,h,nTrials,spec,opt,C1,C2,tailFrac)
%% initialize
garch_fit = cell(n,1);
margins   = cell(n,1);
residuals = NaN(size(X,1),n);
variances = NaN(size(X,1),n);
E         = NaN(size(X,1),n);
U         = zeros(size(residuals));
parT      = zeros(n,3);
parT_all  = [];
%parN      = zeros(n,2);

%% generate U(0,1) data
for b = 1:n % for each firm + market
    %% data check
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if max(isnan(X(:,b)))
        continue;
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% ARMA-GARCH estimation
    garch_fit{b} = estimate(spec, X(:,b),'Display','off','options',opt);
    
    [residuals(:,b), variances(:,b)] = infer(garch_fit{b}, X(:,b));
    
    % Having filtered the model residuals from each return series, standardize
    % the residuals by the corresponding conditional standard deviation. These
    % standardized residuals represent the underlying zero-mean, unit-variance,
    % i.i.d. series upon which the EVT estimation of the sample CDF tails is based.
    E(:,b) = residuals(:,b)./sqrt(variances(:,b));
    
    %% fit marginal distributions
    % 1. fit standardized residuals with univariate t distribution
    opt             = statset('mlecustom');
    opt.MaxIter     = 333;  % Avoid warning message: mle did not converge. Iteration limit exceeded
    opt.MaxFunEvals = 1000; % ...
    
    parT(b,:)   = mle(E(:,b),'distribution','tlocationscale','opt',opt);
    % normalize parameters
    parT(b,2)   = max(3,parT(b,2));   % The variance is only defined for nu >= 3
    parT(b,3)   = min(200,parT(b,3)); % set nu <= 200
    parT_all    = [parT_all; parT(b,:)];
    margins{b}  = makedist('tlocationscale','mu',parT(b,1),'sigma',parT(b,2),'nu',parT(b,3));
    
    % 2. fit standardized residuals with univariate normal distribution (Can't be used here! coz we use t-copula)
%     parN(b,:)  = mle(E(:,b));
%     margins{b} = makedist('Normal','mu',parN(b,1),'sigma',parN(b,2));

    
    % 3. fit standardized residuals with parametric GP tails and nonparametric kernel-smoothed interior
%     margins{b} = paretotails(E(:,b), tailFrac, 1 - tailFrac, 'kernel');
    
    %%%%%%%%%%%%%%%%%% F(X) --- U(0,1) %%%%%%%%%%%%%%%%%%%%%%%%%
    % compute the CDF for Univariate t distribution, evaluated at the values in E(:,b)
    % (Note that we use IFM here, not CML !!!)
    % 1. Canonical maximum likelihood (CML): Uniform variates are transformed by the empirical CDF of each margin.
    % 2. Inference Functions for Margins (IFM): It estimated the marginal distributions first, U contains data 
    %    transformed to the unit hypercube by parametric estimates of their marginal CDF.
    
    %% probability transformation (transform margin to standard uniform)
    U(:,b) = cdf(margins{b},E(:,b)); 
end
%% estimate t copula and generate nTrials random vectors from the fitted copula
% 1. use Kresta's code
%Uf = copula_estsim(U,'student', nTrials, h);

% model the dependence structure among standardized residuals
% 2. use built in function 'copulafit'
%[R,DoF] = copulafit('t',U,'Method','ApproximateML'); % fit t-copula

% 3. use the method of moments to fit t-copula
[R,DoF] = tcopulafit(U);

% Add these two lines if you need reproductive results!
%s = RandStream.getGlobalStream();
%reset(s)

%% Copula simulation
Z = zeros(h,nTrials,n);               % standardized residuals array
U = copularnd('t',R,DoF,h * nTrials); % t copula simulation

%% Quantile transformation
for j = 1:n
    Z(:,:,j) = reshape(icdf(margins{j}, U(:,j)), h, nTrials);
end

%% generate return scenarios
Y0 = X(end,:);           % presample returns
Z0 = E(end,:);           % presample standardized residuals
V0 = variances(end,:);   % presample variances

simulated = zeros(h,nTrials,n);

for i = 1:n
    simulated(:,:,i) = filter(garch_fit{i}, Z(:,:,i), 'Y0', Y0(i), 'Z0', Z0(i), 'V0', V0(i));
end

% Now reshape the simulated returns array such that each page represents a 
% single trial of a multivariate return series, rather than multiple trials of
% a univariate return series.
simulated = permute(simulated, [1 3 2]); % [h n nTrials]

% Since we are working with daily logarithmic returns, the 
% cumulative returns over the forcast horizon are simply the sums of the returns 
% over each intervening period.

% Note that although the simulated returns are logarithmic (continuously 
% compounded), the portfolio optimization problem is usually solved using simple returns,
% so we convert the cumulative log returns back to arithmetic returns.
% (...proper procedure is to convert the log returns to arithmetic returns for the purposes of 
%     portfolio optimization and risk management.)

%% Calculate Multi-period arithmetic return
multi_ret  = zeros(nTrials,n);
for r1 = 1:n
    for r2 = 1:nTrials
        multi_ret(r2,r1) = exp(sum(simulated(:,r1,r2))) - 1;
    end
end 

%% Calculate truncated return distributions
% first column is the market
rm         = multi_ret(:,1);
% obtain truncated returns of all firms and the market
trun_ret1  = multi_ret(rm<C1,:);
trun_ret2  = multi_ret(rm<C2,:);
%{
for b=1:size(returns,2)
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if max(isnan(returns(t-h-m:t-h-1,b)))
        sim_ret(b,:)=-1;
        continue;
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%% F^-1(U) --- F %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Uf(:,b) = icdf('tlocationscale',Uf(:,b),parT(b,1),parT(b,2),parT(b,3)); % quantile transformation
    
    % numObs: number of observations (rows) to generate for each path of the return series, here we generate one-day ahead return so numObs = 1
    % NumPaths: number of sample paths
    % Uf(:,b)' : (1 * NumPaths)
    sim_ret(b,:) = filter(parameters(b), Uf(:,b)', 'Y0', Y0(b), 'Z0', Z0(b), 'V0', V0(b));
end
simulated = sim_ret'; % simulated log returns obtained from AR(1)-GJR_GARCH(1,1)-t Copula model
%}
end