function [multi_ret,trun_ret1,trun_ret2] = dcc_sim(X,n,T,h,s,C1,C2)
%% Demean returns without using AR(1)
%X = X - ones(size(X,1),1)*mean(X); % demeaned returns (centered data)

%% AR(1) estimation
AR_para = zeros(2*n,1);
f_res   = zeros(T,n);
for i = 1:n
    [parameters, ~, res] = armaxfilter(X(:,i),1,1);
    AR_para(2*i-1:2*i,1) = parameters;
    f_res(:,i)           = res;
end

%% Fit GJR-GARCH-DCC model
[para, ~, ~, Rt, Qt, stdresid, ~, ~, ~, ~, ~, H] = dcc_mvgarch(f_res,1,1,1,1);
%% Initialize
% Ut is the inverse of lower-triangular matrix in the Cholesky decomposition of the correlation matrix
Ut    = cell(T,1);
% Standardized GARCH-DCC innovations
st_in = zeros(n,T);

%% Cholesky decomposition of covariance matrix
% Note that the conditional correlation is also the conditional covariance between standardized disturbances (volatility adjusted returns)
% The correlation matrix Rt is modelled for volatility adjusted returns
for i1 = 1:T
    Ut{i1}      = inv(chol(Rt(:,:,i1))'); % inverse of lower triangular matrix Lt = Ut'
    st_in(:,i1) = Ut{i1} * stdresid(i1,:)'; % obtain uncorrelated standardized innovations
end

%% Nonparametric Bootstrap (sample with replacement s*h vectors of standardized innovations)
% construct s pseudo sample of GARCH-DCC standardized innovations over horizon h
st_inno = zeros(n,h,s);
for i2 = 1:s
    % Generate a 1-by-h row vector of uniformly distributed random integers between 1 and T
    c               = ceil(rand(1,h)*(size(stdresid,1)-1))+1;
    st_inno(:,:,i2) = st_in(:,c);
end

%% DCC parameters to calculate h correlation matrices for each pseudo sample
% volatility adjusted returns (standardized residuals on [1,T])
eps    = stdresid';
% alpha and beta are estimated parameters of DCC models
alpha  = para(end-1);
beta   = para(end);

% Compute the unconditional correlation matrix of the firm and market adjusted returns,
% which is equal to the unconditonal covariance matrix of volatility adjusted returns
Qbar      = cov(stdresid);
% in order to estimate the constant omega in DCC, we use variance targeting, i.e., omega = Qbar*(1-alpha-beta)
omega_dcc = (1-alpha-beta) * Qbar;

Q       = cell(s,h);
R       = cell(s,h);
eps_s   = zeros(n,h,s);

for q = 1:s
    % use the last value of the correlation matrix QT as initial condition
    Q{q,1}       = omega_dcc + alpha * (eps(:,end) * eps(:,end)') + beta * Qt(:,:,end);
    R{q,1}       = Q{q,1}./(sqrt(diag(Q{q,1}))*sqrt(diag(Q{q,1}))');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % predicted standardized residuals
    eps_s(:,1,q) = chol(R{q,1})' * st_inno(:,1,q); % Lt * (pseudo samples of standardized innovations)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for qq = 1:h-1
        Q{q,qq+1}       = omega_dcc + alpha * (eps_s(:,qq,q) * (eps_s(:,qq,q))') + beta * Q{q,qq};
        R{q,qq+1}       = Q{q,qq+1}./(sqrt(diag(Q{q,qq+1}))*sqrt(diag(Q{q,qq+1}))');
        eps_s(:,qq+1,q) = chol(R{q,qq+1})' * st_inno(:,qq+1,q);
    end
end

%% predict conditional variances (bootstrap)
con_var = zeros(n,h,s);
log_ret = zeros(n,h,s);
errors  = zeros(n,h,s);

for i3 = 1:n
    % the total number of parameters is 3*(n+1) + 2, last two are DCC parameters
    omega = (1 - para(3*i3-2) - 0.5*para(3*i3-1) - para(3*i3)) * var(f_res(:,i3));
    %omega = (1 - para(3*i3-2) - 0.5*para(3*i3-1) - para(3*i3)) * var(X(:,i3));
    for m = 1:s
        % use the last value of conditional variance at T as initial condition
        con_var(i3,1,m) = omega + para(3*i3-2) * f_res(end,i3)^2 + para(3*i3-1) * f_res(end,i3)^2 * (f_res(end,i3)<0)...
                          + para(3*i3) * H(end,i3);
        
        errors(i3,1,m)  = eps_s(i3,1,m) * sqrt(con_var(i3,1,m));
        
        log_ret(i3,1,m) = errors(i3,1,m) + AR_para(2*i3-1,1)+AR_para(2*i3,1)*X(end,i3);
        
        %con_var(i3,1,m) = omega + para(3*i3-2) * X(end,i3)^2 + para(3*i3-1) * X(end,i3)^2 * (X(end,i3)<0)...
         %   + para(3*i3) * H(end,i3); 
        %log_ret(i3,1,m) = eps_s(i3,1,m) * sqrt(con_var(i3,1,m));
        for ii = 1:h-1
            con_var(i3,ii+1,m) = omega + para(3*i3-2) * errors(i3,ii,m)^2 + para(3*i3-1) * errors(i3,ii,m)^2 * ...
                                 (errors(i3,ii,m)<0) + para(3*i3) * con_var(i3,ii,m);
            
            errors(i3,ii+1,m)  = eps_s(i3,ii+1,m) * sqrt(con_var(i3,ii+1,m));
            
            log_ret(i3,ii+1,m) = errors(i3,ii+1,m) + AR_para(2*i3-1,1)+AR_para(2*i3,1)*log_ret(i3,ii,m);
            
            %con_var(i3,ii+1,m) = omega + para(3*i3-2) * log_ret(i3,ii,m)^2 + para(3*i3-1) * log_ret(i3,ii,m)^2 * ...
             %   (log_ret(i3,ii,m)<0) + para(3*i3) * con_var(i3,ii,m);
            %log_ret(i3,ii+1,m) = eps_s(i3,ii+1,m) * sqrt(con_var(i3,ii+1,m));
        end
    end 
end

%% Calculate arithmetic multi-period returns for each sample
% The first column is market
multi_ret  = zeros(s,n);
for i4 = 1:n
    for i11 = 1:s
        multi_ret(i11,i4) = exp(sum(log_ret(i4,:,i11))) - 1;
    end
end

%% Calculate truncated return distributions
% first column refers to market index
rm         = multi_ret(:,1);
% truncated returns of firms and the market
trun_ret1  = multi_ret(rm<C1,:);
trun_ret2  = multi_ret(rm<C2,:);
end