% ROLLING WINDOW METHOD
%
% Generate monthly simulated simple returns for all firms and the market index
%
% Use either Copula or DCC to model the dependence structure
% 1. Copula: innovations have a t distribution
% 2. DCC: returns have unspecified multivariate distribution with zero mean and time-varying covariance
%
% Date: 05/02/2023

%% Create simulated data
clear
close all
warning('off','all');
% addpath(genpath(pwd))

% fid = fopen('ticker.txt','w');
% fprintf(fid, '%s\n',ticker(1:end-1));
% fprintf(fid,'%s',ticker(end));
% fclose(fid);

% read tickers of 23 assets
fileName = 'ticker.txt';
FID = fopen(fileName);
data = textscan(FID,'%s');
fclose(FID);
ticker = string(data{:});

% writematrix(returns,'returns.txt')

% read dates
dates = readmatrix('dates.txt');
% read prices (1st col -> market)
prices = readmatrix('prices.txt');

%% load firm names and tickers

% 16 SIFIs + 7 non-SIFIs (23 in total)
load('both.mat')
all_ticker = table2array(both(:,1));
all_name = table2array(both(:,2));
all_cusip = table2array(both(:,3));
load both_prc % load prices and dates for both SIFIs and non-SIFIs

returns      = price2ret(prices);      % convert to log returns
T            = size(returns,1);        % # of returns (i.e., historical sample size)
dates_prices = dates;                  % dates of prices
dates        = dates(2:end);           % dates of returns
nTrials      = 30000;                  % sample size of simulation
m            = 1500;                   % window size
h            = 22;                     % forecast horizon
n            = size(returns,2);        % # of time series(including market index)
tailFrac     = 0.1;                    % Decimal fraction of residuals allocated to each tail

% Choose two specifications for the threshold C (for robustness check)
% to see differences in final results under milder or stronger definitions of systemic risk.
C1 = 0;      % when the market index experiences negative returns
C2 = -0.067; % (-6.7%) corresponds to 40% decrease in the market index over 6-month

%%
% specify AR(1)-GJR(1,1)
opt  = optimoptions(@fmincon, 'Display', 'off', 'Diagnostics', 'off', 'Algorithm', 'sqp', 'TolCon', 1e-7);
spec = arima('AR',NaN,'Distribution','t','Variance',gjr(1,1));  

% The last day of every month contained in the backtesting period
smpl = [];
for year = 2007:2020
    for i = 1:12
        smpl = [smpl; datenum(sprintf('%d%02d%d',year,i,eomday(year,i)) , 'yyyymmdd')];
    end
end

%% Rolling window estimation
% cache the chunk of sliced data
H = cell(length(smpl),1);
% cache the # of return observations before the end of each month
t = zeros(length(smpl),1);
for b = 1:length(smpl)
    t(b) = find( dates <= smpl(b) , 1, 'last');
    H{b} = returns(t(b)-h-m+1:t(b)-h,:);
end

%% Run the rolling window estimation using parallel
% copula/dcc_simulated: simulated return scenarios
% copula/dcc_c1:        simulated return scenarios truncated by the threshold C=0
% copula/dcc_c2:        simulated return scenarios truncated by the threshold C=-6.7%
tic
parfor a = 1:length(smpl)
    
    [copula_simulated, copula_c1, copula_c2] = copula_sim(H{a},n,h,nTrials,spec,opt,C1,C2,tailFrac);
    
    [dcc_simulated, dcc_c1, dcc_c2] = dcc_sim(H{a},n,m,h,nTrials,C1,C2);
    
    parsave(['sim_' int2str(smpl(a)) '.mat'],copula_simulated,dcc_simulated,copula_c1,dcc_c1,copula_c2,dcc_c2);
    
end
toc